<?php

require_once(plugin_dir_path(__FILE__) . 'xwp.php');

class PostManager {

    // Properties

    //constructor
    public function __construct() {
        
    }

    public function register_apis() {
        global $_xwp;
        $_xwp->register_route('add-post', array($this,'add_post'));
        $_xwp->register_route('get-posts', array($this,"get_posts"));
        $_xwp->register_route('get-post', array($this,"get_post"));
        $_xwp->register_route('delete-post', array($this,"delete_post"));
        $_xwp->register_route('update-post', array($this,"update_post"));
        $_xwp->register_route('add-media', array($this,"add_media"));
        $_xwp->register_route('set-post-thumbnail', array($this,"set_post_thumbnail"));
    }


    private function get_post_array(WP_REST_Request $request) {
        $title = sanitize_text_field($request->get_param('_title'));
        $content = $request->get_param('_content');
        $author = $request->get_param('_author'); // Optional: you can handle this differently
        $status = $request->get_param('_status');
        $categories = $request->get_param('_categories');
        $tags = $request->get_param('_tags');
        $excerpt = $request->get_param('_excerpt');
        // Create a new post
        $new_post = array(
            'post_title'    => $title,
            'post_content'  => $content,
            'post_status'   => $status ? $status : 'draft', // Default to draft
            'post_author'   => $author ? $author : 1, // Default to admin user
            'post_type'     => 'post',
            'post_status'   => $status ? $status : 'draft', // Default to draft
            'post_category' => $categories ? $categories : array(1), // Default to uncategorized
            'tags_input'    => $tags ? $tags : '',
            'post_excerpt'  => $excerpt ? $excerpt : '',

        );

        return $new_post;

    }


    public function add_post(WP_REST_Request $request) {
        
        
        $new_post = $this->get_post_array($request);

        // Insert the post into the database
        $post_id = wp_insert_post($new_post);

        if ($post_id) {
            // return json response
            $res = array(
                '_post_id' => $post_id,
                '_message' => 'Article added successfully'
            );
            return new WP_REST_Response($res, 200);
        } else {
            return new WP_REST_Response('Failed to add article', 500);
        }
    }
    // add media to media library
    public function add_media(WP_REST_Request $request) {
        $image_base64 = $request->get_param('_image_url');
        // error_log('image_base64: ' . $image_base64);
        $decoded_image = base64_decode($image_base64);
    if ($decoded_image === false) {
        return new WP_REST_Response('Failed to decode image', 400);
    }

    // Get the WordPress upload directory
    $upload_dir = wp_upload_dir();
    $file_name =   'image' . time() . '.jpg';
    $file_path = $upload_dir['path'] . '/' . sanitize_file_name($file_name);

    // Save the decoded image as a file
    file_put_contents($file_path, $decoded_image);

    // Check if the file was successfully created
    if (!file_exists($file_path)) {
        return new WP_REST_Response('Failed to save file', 500);
    }

    $file_type = wp_check_filetype($file_name)['type'];
    // Prepare an array of post data for the attachment
    $attachment = array(
        'guid' => $upload_dir['url'] . '/' . basename($file_path),
        'post_mime_type' => $file_type,
        'post_title' => sanitize_file_name($file_name),
        'post_content' => '',
        'post_status' => 'inherit'
    );

    // Insert the attachment into the database
    $attachment_id = wp_insert_attachment($attachment, $file_path);

    // If the attachment was successfully created
    if ($attachment_id) {
        // Generate attachment metadata and update the database record
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        $attach_data = wp_generate_attachment_metadata($attachment_id, $file_path);
        wp_update_attachment_metadata($attachment_id, $attach_data);

        // Return success response with attachment ID
        $res = array(
            '_attachment_id' => $attachment_id,
            '_message' => 'Image uploaded successfully from Base64'
        );
        return new WP_REST_Response($res, 200);
    } else {
        return new WP_REST_Response('Failed to upload image', 500);
    }
}


public function set_post_thumbnail($post_id, $thumbnail_id) {
    // Ensure that we have a valid post ID and thumbnail ID
    $post = get_post($post_id);
    $thumbnail_id = absint($thumbnail_id);

    // Check if the post and thumbnail are valid
    if ($post && $thumbnail_id && get_post($thumbnail_id)) {
        // Check if the thumbnail ID is a valid image attachment
        if (wp_get_attachment_image($thumbnail_id, 'thumbnail')) {
            // Set the featured image (thumbnail) for the post
            return update_post_meta($post->ID, '_thumbnail_id', $thumbnail_id);
        } else {
            // If the thumbnail is not valid, remove the existing thumbnail
            return delete_post_meta($post->ID, '_thumbnail_id');
        }
    }

    // Return false if the post or thumbnail is not valid
    return false;
}


    public function get_posts(WP_REST_Request $request) {

        $pid = $request->get_param('_id');
        $args = array(
            'post_type' => 'post',
            'posts_per_page' => -1,
        );

        if ($pid) {
            $args['p'] = $pid;
        }

        
        
        $query = new WP_Query($args);
        $posts = $query->posts;

        $response = array();
        foreach ($posts as $post) {
            $response[] = $post->to_array();
        }

        return new WP_REST_Response($response, 200);
    }

    public function delete_post(WP_REST_Request $request) {
        $post_id = $request->get_param('_id');
        $result = wp_delete_post($post_id);

        if ($result) {
            return new WP_REST_Response('Article deleted successfully', 200);
        } else {
            return new WP_REST_Response('Failed to delete article', 500);
        }
    }

    public function update_post(WP_REST_Request $request) {
        $post_id = $request->get_param('_id');
        $post_data = $this->get_post_array($request);
        $post_data['ID'] = $post_id;
        $result = wp_update_post($post_data);
        // error_log(print_r($result, true));
        if ($result) {
            return new WP_REST_Response('Article updated successfully', 200);
        } else {
            return new WP_REST_Response('Failed to update article', 500);
        }
    }

    
}

?>
